// Quelle des Programmes: https://www.instructables.com/id/Neo-Pixel-Arduino-Vu-Meter/
// Kommentierungen und Anpassungen durch Marc Scholl und Alexander Pusch

#include <Adafruit_NeoPixel.h> // Bibliothek wird eingefügt 
  
#define N_PIXELS 60         // Anzahl der gesamten Pixel im Strang 
#define MIC_PIN A5          // Das Mikrofon ist an diesen Analog-Pin angeschlossen 
#define LED_PIN 6           // NeoPixel LED-Strang ist an diesen Pin angeschlossen 
#define DC_OFFSET 0         // DC-Offset im Mic-Signal - wenn ungewiss, lassen Sie 0 
#define NOISE 20            // Geräusche / Brummen / Störungen im Mic-Signal 
#define SAMPLES 40          // Länge des Pufferspeichers für die dynamische Pegeleinstellung 
#define TOP (N_PIXELS + 2)  // Lassen Sie den Punkt etwas von der Skala ab 
#define PEAK_FALL 60        // Rate des fallenden Spitzenpunktes 
  
  
byte 
  peak      = 0,        // Wird für fallenden Punkt verwendet 
  dotCount  = 0,        // Framezähler für die Verzögerung der fallenden Geschwindigkeit 
  volCount  = 0;        // Framezähler zur Speicherung vergangener Volumendaten 
int 
  vol[SAMPLES],         // Sammlung früherer Volumenproben 
  lvl       = 20,       // Aktueller "gedämpfter" Audiopegel 
  minLvlAvg = 0,        // Zur dynamischen Einstellung des Graphen Low & High 
  maxLvlAvg = 512; 
Adafruit_NeoPixel 
  strip = Adafruit_NeoPixel(N_PIXELS, LED_PIN, NEO_GRB + NEO_KHZ800); // sehr wichtig: LED-Strip angeschlossen also Bibliothek verwenden 
 
  
  
void setup() { 
  
  Serial.begin(9600);   // Start für seriellen Monitor (Plotter hier besser) 
  
analogReference (EXTERNAL); 
  
  memset(vol, 0, sizeof(vol)); 
  strip.begin(); 
} 


  
void loop() { 
  uint8_t  i; 
  uint16_t minLvl, maxLvl; 
  int n, height; 
  n   = analogRead(MIC_PIN);                  // Reines Lesen vom Mikrofon 
  Serial.println(n);                          // Ausgabe im seriellen Monitor 
  n   = abs(n - 512 - DC_OFFSET);             // Auf Null einstellen 
  n   = (n <= NOISE) ? 0 : (n - NOISE);       // Rauschen / Brummen entfernen 
  lvl = ((lvl * 7) + n) >> 3;                 // "Gedämpftes" Lesen (sonst siehts verwirrend aus) 
  
  
  
  height = TOP * (lvl - minLvlAvg) / (long)(maxLvlAvg - minLvlAvg); 
  
  if(height < 0L)       height = 0;      // Clip Ausgabe 
  else if(height > TOP) height = TOP; 
  if(height > peak)     peak   = height; // Haelt den maximalen (Peak)Punkt oben 
  
  
  // Farbpixel basierend auf Regenbogengefaelle 
  for(i=0; i<N_PIXELS; i++) { 
    if(i >= height)               strip.setPixelColor(i,   0,   0, 0); 
    else strip.setPixelColor(i,Wheel(map(i,0,strip.numPixels()-1,30,150))); 
   
  } 
  
  
  
  // Zeichne den Spitzenpunkt 
  if(peak > 0 && peak <= N_PIXELS-1) strip.setPixelColor(peak,Wheel(map(peak,0,strip.numPixels()-1,30,150))); 
  
   strip.show(); // Update Strip 
  
  // Alle paar Frames machen den Peak-Pixel um 1: 
  
    if(++dotCount >= PEAK_FALL) { //Fallrate 
     
     if(peak > 0) peak--; 
     dotCount = 0; 
    } 
  
  
  
  vol[volCount] = n;                      // gespeichertes Beispiel f�r dynamisches Nivellieren speichern 
  if(++volCount >= SAMPLES) volCount = 0; // Vorschub / Rollover-Sample-Z�hler 
  
  
  minLvl = maxLvl = vol[0]; 
  for(i=1; i<SAMPLES; i++) { 
    if(vol[i] < minLvl)      minLvl = vol[i]; 
    else if(vol[i] > maxLvl) maxLvl = vol[i]; 
  } 
  
  if((maxLvl - minLvl) < TOP) maxLvl = minLvl + TOP; 
  minLvlAvg = (minLvlAvg * 63 + minLvl) >> 6; // Dämpfen min/max Level 
  maxLvlAvg = (maxLvlAvg * 63 + maxLvl) >> 6; // gefälschter fortlaufender Durchschnitt 
  
} 
  
// Geben Sie einen Wert von 0 bis 255 ein, um einen Farbwert zu erhalten. 
// Die Farben sind ein übergang von r - g - b - zurück zu r. 
uint32_t Wheel(byte WheelPos) { 
  if(WheelPos < 85) { 
   return strip.Color(WheelPos * 3, 255 - WheelPos * 3, 0); 
  } else if(WheelPos < 170) { 
   WheelPos -= 85; 
   return strip.Color(255 - WheelPos * 3, 0, WheelPos * 3); 
  } else { 
   WheelPos -= 170; 
   return strip.Color(0, WheelPos * 3, 255 - WheelPos * 3); 
  } 
} 

